@@ -1659,6 +1659,13 @@ rdp_peer_capabilities (freerdp_peer *peer)
       session_rdp->monitor_config->monitor_count = 1;
     }
 
+  if (rdp_settings->SupportGraphicsPipeline || rdp_settings->RemoteFxCodec ||
+      rdp_settings->NSCodec)
+    {
+      g_debug ("[RDP] Fixing invalid colour depth set by client");
+      rdp_settings->ColorDepth = 32;
+    }
+
   switch (rdp_settings->ColorDepth)
     {
     case 32:
@@ -1860,9 +1867,9 @@ rdp_peer_context_new (freerdp_peer   *peer,
   rdp_peer_context->planar_flags |= PLANAR_FORMAT_HEADER_RLE;
 
   rdp_peer_context->vcm = WTSOpenServerA ((LPSTR) peer->context);
-  if (!rdp_peer_context->vcm)
+  if (!rdp_peer_context->vcm || rdp_peer_context->vcm == INVALID_HANDLE_VALUE)
     {
-      g_warning ("[RDP] Failed to retrieve VCM handle");
+      g_warning ("[RDP] Failed to create virtual channel manager");
       rdp_peer_context_free (peer, rdp_peer_context);
       return FALSE;
     }
@@ -2138,13 +2145,21 @@ grd_session_rdp_new (GrdRdpServer      *rdp_server,
   username = grd_settings_get_rdp_username (settings, &error);
   if (!username)
     {
-      g_warning ("Couldn't retrieve RDP username: %s", error->message);
+      if (error)
+        g_warning ("[RDP] Couldn't retrieve RDP username: %s", error->message);
+      else
+        g_message ("[RDP] Username is not set, denying client");
+
       return NULL;
     }
   password = grd_settings_get_rdp_password (settings, &error);
   if (!password)
     {
-      g_warning ("Couldn't retrieve RDP password: %s", error->message);
+      if (error)
+        g_warning ("[RDP] Couldn't retrieve RDP password: %s", error->message);
+      else
+        g_message ("[RDP] Password is not set, denying client");
+
       g_free (username);
       return NULL;
     }
@@ -2180,6 +2195,21 @@ grd_session_rdp_new (GrdRdpServer      *rdp_server,
   return g_steal_pointer (&session_rdp);
 }
 
+static void
+clear_session_sources (GrdSessionRdp *session_rdp)
+{
+  if (session_rdp->update_monitor_layout_source)
+    {
+      g_source_destroy (session_rdp->update_monitor_layout_source);
+      g_clear_pointer (&session_rdp->update_monitor_layout_source, g_source_unref);
+    }
+  if (session_rdp->pending_encode_source)
+    {
+      g_source_destroy (session_rdp->pending_encode_source);
+      g_clear_pointer (&session_rdp->pending_encode_source, g_source_unref);
+    }
+}
+
 static gboolean
 clear_pointer_bitmap (gpointer key,
                       gpointer value,
@@ -2239,12 +2269,7 @@ grd_session_rdp_stop (GrdSession *session)
   g_clear_object (&rdp_peer_context->graphics_pipeline);
 
   g_clear_pointer (&session_rdp->socket_thread, g_thread_join);
-
-  if (session_rdp->update_monitor_layout_source)
-    {
-      g_source_destroy (session_rdp->update_monitor_layout_source);
-      g_clear_pointer (&session_rdp->update_monitor_layout_source, g_source_unref);
-    }
+  clear_session_sources (session_rdp);
 
   peer->Close (peer);
   g_clear_object (&session_rdp->connection);
@@ -2266,7 +2291,7 @@ grd_session_rdp_stop (GrdSession *session)
   g_hash_table_foreach_remove (session_rdp->pressed_unicode_keys,
                                notify_keysym_released,
                                session_rdp);
-  g_clear_object (&session_rdp->rdp_event_queue);
+  grd_rdp_event_queue_flush (session_rdp->rdp_event_queue);
 
   g_clear_pointer (&session_rdp->rdp_surface, grd_rdp_surface_free);
   g_clear_pointer (&session_rdp->monitor_config, grd_rdp_monitor_config_free);
@@ -2417,15 +2442,13 @@ grd_session_rdp_dispose (GObject *object)
 {
   GrdSessionRdp *session_rdp = GRD_SESSION_RDP (object);
 
-  if (session_rdp->pending_encode_source)
-    {
-      g_source_destroy (session_rdp->pending_encode_source);
-      g_clear_pointer (&session_rdp->pending_encode_source, g_source_unref);
-    }
+  clear_session_sources (session_rdp);
 
   g_assert (!session_rdp->graphics_thread);
   g_clear_pointer (&session_rdp->graphics_context, g_main_context_unref);
 
+  g_clear_object (&session_rdp->rdp_event_queue);
+
   g_clear_pointer (&session_rdp->pressed_unicode_keys, g_hash_table_unref);
   g_clear_pointer (&session_rdp->pressed_keys, g_hash_table_unref);
   g_clear_pointer (&session_rdp->pointer_cache, g_hash_table_unref);
